#!python.exe
import importlib.util
import pathlib
import re
import sys

description = """DocuBox - The swiss army knife for document solution powered by Python.
Copyright (C) 2022,2023 Hiroshi Miura

DocuBox is a single binary utility that have several document solution commands.
When it is called as sphinx-build, it behave as sphinx.
When it is called as mkdocs, it behave as mkdocs.

Please symbolic link to the name you want.

Supported features
"""

commands = [
    # (command name, package, func)
    ("babel", "babel.messages.frontend", "main"),
    ("j2", "j2cli", "main"),
    ("jinja2", "jinja2.ext", "babel_extract"),
    ("jsonschema", "jsonxchema.cli", "main"),
    ("m2r", "m2r", "main"),
    ("markdown_it", "markdown_it.cli.parse", "main"),
    ("mkdocs", "mkdocs.__main__", "cli"),
    ("myst-anchors", "myst_parser.cli", "print_anchors"),
    ("myst-docutils-html", "myst_parser.cli.docutils_", "cli_html"),
    ("myst-docutils-html5", "myst_parser.cli.docutils_", "cli_html5"),
    ("myst-docutils-latex", "myst_parser.cli.docutils_", "cli_latex"),
    ("myst-docutils-pseudoxml", "myst_parser.cli.docutils_", "cli_pseudoxml"),
    ("myst-docutils-xml", "myst_parser.cli.docutils_", "cli_xml"),
    ("pygmentize", "pygments.cmdline", "main"),
    ("sphinx-apidoc", "sphinx.ext.apidoc", "main"),
    ("sphinx-autogen", "sphinx.ext.autosummary.generate", "main"),
    ("sphinx-build", "sphinx.cmd.build", "main"),
    ("sphinx-quickstart", "sphinx.cmd.quickstart", "main"),
    ("sphinx-intl", "sphinx_intl.commands", "main"),
]


def main():
    cmdname = pathlib.Path(sys.argv[0]).name.lower()
    for cmd in commands:
        if cmdname == cmd[0]:
            try:
                mod = importlib.import_module(cmd[1])
                func = getattr(mod, cmd[2])
                return(func())
            except ModuleNotFoundError as e:
                print(e.msg)
                break
    # unknown command or got exception
    print(description)
    for cmd in commands:
        print(f"- {cmd[0]}")
    return(1)


if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
    sys.exit(main())



submodules = [
    "babel",
    "charset_normalizer",
    "datautil",
    "myst-parser",
    "sphinx",
    "sphinx-intl",
    "sphinxemoji",
    "markdown",
    "markdown-it",
    "linkify-it",
    "mdit_py_plugins",
    "myst_parser",
    "openpyxl",
    "pygments",
    "snowballstemmer",
    "alabaster",
    "sphinx_bootstrap_theme",
    "sphinx_rtd_theme",
    "mkdocs_material",
    "lunr",
    # "sphinx_needs",  ## exclude
]
hiddenimports = [
    "certifi",
    "click",
    "idna",
    "inspect",
    "locale",
    "pkg_resources",
    "importlib_metadata",
    "markupsafe",
    "pytz",
    "pyrsistent",
    "yaml",
    "materialx",
    "material",
    "pymdownx",
    "mkdocsplantuml",
    "mkdocs_alabaster",
    "mkdocs_badges",
    "mkdocs_coverage",
    "j2cli",
    "jinja2",
    "ghp_import",
]
collect_all = [
    "docutils",
    "mkdocs",
    "mkdocs_custommill",
    "sphinx",
]
metadata = [
    "Babel",
    "markdown",
    "mkdocs",
    "mkdocs_alabaster",
    "mkdocs_material",
    "mkdocs_plantuml",
    "mkdocs_badges",
    "mkdocs_coverage",
    "alabaster",
    "sphinx_bootstrap_theme",
]
package_data = [
    "babel",
    "certifi",
    "charset_normalizer",
    "jsonschema",
    "pytz",
    "markdown_it",
    "mdit_py_plugins",
    "alabaster",
    "mkdocs_alabaster",
    "mkdocs_badges",
    "sphinx_bootstrap_theme",
    "sphinx_rtd_theme",
    "material",
    "pygments_style_kit",
    "sphinxcontrib"
    # "sphinx_needs",
]
excludes = [
    'sphinx.testing',
    'sphinx.testing.comparer',
    'sphinx.testing.fixtures',
    'sphinx.testing.path',
    'sphinx.testing.restructuredtext',
    'sphinx.testing.util',
    'mkdocs.tests',
    'mkdocs.tests.base',
    'mkdocs.tests.build_tests',
    'mkdocs.tests.cli_tests',
    'mkdocs.tests.config',
    'mkdocs.tests.config.base_tests',
    'mkdocs.tests.config.config_options_legacy_tests',
    'mkdocs.tests.config.config_options_tests',
    'mkdocs.tests.config.config_tests',
    'mkdocs.tests.gh_deploy_tests',
    'mkdocs.tests.integration',
    'mkdocs.tests.livereload_tests',
    'mkdocs.tests.localization_tests',
    'mkdocs.tests.new_tests',
    'mkdocs.tests.plugin_tests',
    'mkdocs.tests.search_tests',
    'mkdocs.tests.structure',
    'mkdocs.tests.structure.file_tests',
    'mkdocs.tests.structure.nav_tests',
    'mkdocs.tests.structure.page_tests',
    'mkdocs.tests.structure.toc_tests',
    'mkdocs.tests.theme_tests',
    'mkdocs.tests.utils',
    'mkdocs.tests.utils.babel_stub_tests',
    'mkdocs.tests.utils.utils_tests',
    'numpy',
    'six',
    'toml',
    'unittest',
    'email',
    'setuptools',
    'distutils',
    'pygls',
    'matplotlib',
    'pyparsing',
    'PIL',
]


#!/usr/bin/env python
import sys

from .version import __version__

__copyright__ = "Copyright (C) 2022,2023 Hiroshi Miura"


if __name__ == "__main__":
    from . import launcher
    sys.exit(launcher.main())



import argparse
import hashlib
import importlib.machinery
import importlib.metadata
import importlib.util
import os
import pathlib
import sys
from typing import Any, Optional

from . import config
from . import launcher as _docubox

PROJECT_NAME="docubox"
DIST_DIR = pathlib.Path("dist")
LAUNCHER = "launcher.py"

def is_64bit() -> bool:
    """check if running platform is 64bit python."""
    return sys.maxsize > 1 << 32


def build(name: str, sha: bool, ln: bool, loglevel: str) -> int:
    # PyInstaller arguments
    pyinstaller_args = [
        "--noconfirm",
        "--console",
        "--onefile",
        "--name", name
    ]

    add_arg = "{src:s}" + os.pathsep + "{name:s}"
    for mod in config.collect_all:
        pyinstaller_args.append("--collect-all")
        pyinstaller_args.append(mod)
    for mod in config.package_data:
        origin = importlib.util.find_spec(mod).origin
        if origin is not None:
            pyinstaller_args.append("--add-data")
            pyinstaller_args.append(add_arg.format(src=os.path.dirname(origin), name=mod))
    for mod in config.hiddenimports:
        pyinstaller_args.append("--hidden-import")
        pyinstaller_args.append(mod)
    for sub in config.submodules:
        pyinstaller_args.append("--collect-submodules")
        pyinstaller_args.append(sub)
    for pkg in config.metadata:
        pyinstaller_args.append("--copy-metadata")
        pyinstaller_args.append(pkg)
    for mod in config.excludes:
        pyinstaller_args.append("--exclude-module")
        pyinstaller_args.append(mod)

    # log level default: INFO, can be DEBUG, WARN
    pyinstaller_args.append("--log-level")
    if loglevel is None:
        pyinstaller_args.append("INFO")
    else:
        pyinstaller_args.append(loglevel)
    # give launcher script
    pyinstaller_args.append(os.path.join(os.path.dirname(__file__), LAUNCHER))
    # launch PyInstaller
    from PyInstaller import __main__ as pyinstaller
    pyinstaller.run(pyinstaller_args)
    #
    if sys.platform.startswith("win"):
        # .exe is automatically added on windows
        genfile = name + ".exe"
    else:
        genfile = name
    # Generate sha
    if sha:
        gen_sha(genfile)
    # generate links
    if ln:
        make_link(genfile)
    return 0


def gen_sha(file: str):
    _gen_sha(file, "sha256")
    _gen_sha(file, "sha512")


def _gen_sha(file: str, type: str):
    hash = hashlib.new(type)
    generated = DIST_DIR / file
    with generated.open(mode="rb") as bin:
        hash.update(bin.read())
    hashfile = DIST_DIR / f"{file}.{type}"
    with hashfile.open(mode="w") as f:
        f.write(hash.hexdigest())


def make_link(file: str):
    if sys.platform.startswith("win"):
        # FIXME
        return 0
    # make links for linux and mac
    _make_link(file, PROJECT_NAME)
    for cmd in _docubox.commands:
        _make_link(file, cmd[0])

def _make_link(file, target):
    target_path = pathlib.Path("dist", target)
    target_path.unlink(missing_ok=True)
    os.symlink(file, target_path)


def main(arg: Optional[Any] = None) -> int:
    parse = argparse.ArgumentParser(prog="docubox", description="Standalone binary builder of DocuBox - the swiss army knife for document solution")
    parse.add_argument("-l", "--link", action="store_true", help="Put symbolic links of supported features.")
    parse.add_argument("-n", "--name", nargs=1, help="Specify generated binary name.")
    parse.add_argument("-s", "--sha", action="store_true", help="Generate SHA256 and SHA512 hash files.")
    parse.add_argument("--log-level", nargs=1, help="Specify log level, WARN, INFO, or DEBUG", default="INFO")
    args = parse.parse_args(arg)
    #
    if args.name is not None:
        name = args.name
    else:
        if sys.platform.startswith("win"):
            if is_64bit():
                name = "docubox.windows-x86_64"
            else:
                name = "docubox.windows-x86"
        elif sys.platform.startswith("linux"):
            name = "docubox.linux"
        elif sys.platform.startswith("darwin"):
            name = "docubox.osx"
        else:
            name = "docubox"
    return(build(name, sha=args.sha, ln=args.link, loglevel=args.log_level))


if __name__ == "__main__":
    sys.exit(main(None))
